PROGRAM xmlpaw
  !***************************************************************
  !     Program to take xml output using fsatom.org convention
  !       and create [atom].atomicdata file for pwpaw input
  !       xml output units are Hartree and bohr
  !       pwpaw input files unites are Rydberg and bohr
  !***************************************************************
  USE GlobalMath
  USE atomdata
  USE aeatom
  USE pseudo
  USE basis
  USE xmlword
  IMPLICIT NONE
  CHARACTER (len=4) :: flnm  
  CHARACTER (len=20) :: nm  
  CHARACTER (len=2) :: sym  
  CHARACTER (len=1) :: syml  
  REAL(8), POINTER :: r(:),den(:),rv(:),wfn(:,:)
  INTEGER, POINTER :: n,norbit,nps,npp,npd,npf,npg
  REAL(8), POINTER :: h
  INTEGER, PARAMETER :: VNCT=1,VNCK=2,VSHAPE=3
  CHARACTER(50) :: Vlocaltype
  INTEGER :: Vlocalindex
  INTEGER :: i,j,io,many,l,istart,irc,OK,nbase,ib,ic,icount
  INTEGER :: llmin, llmax, ll,lcount,jcount,id,ie,iop,lp
  REAL(8) :: rc1,rc,x,rr,eself,cc,q00,evale,storeself,qcore,Etotal
  REAL(8) :: tildekin,tildepot,onehat,onehartree,ctexc,ctexc1,cexc1,texc,vexc1
  REAL(8) :: vtexc,vtexc1,oneenergy,ekin,vlocal,tq,fac,stuff,term
  REAL(8) :: ctctse,cthatse,con,q,ecore
  REAL(8), ALLOCATABLE :: ttphi(:),soij(:),stij(:),svij(:)
  REAL(8), ALLOCATABLE :: shij(:),snij(:),dum(:),rh(:),rth(:)
  REAL(8), ALLOCATABLE :: shartree(:),sshartree(:,:,:),wf(:),twf(:)
  REAL(8), ALLOCATABLE :: bm(:),cm(:),dm(:,:)
  INTEGER, PARAMETER :: ifen=11,ifatompaw=12,ifout=13,ifxml=14
  INTEGER, PARAMETER :: maxdim=30
  INTEGER :: lcao_points,lcao_i,iskip,coretailpoints
  REAL(8) :: tphirange,hlcao
  REAL(8), PARAMETER :: coretailtol=1.d-7
  TYPE (GridInfo), TARGET :: AEGrid
  TYPE (PotentialInfo), TARGET :: AEPot
  TYPE (OrbitInfo), TARGET :: AEOrbit
  TYPE (OrbitInfo), TARGET :: PSOrbit
  TYPE (SCFInfo), TARGET :: AESCF
  TYPE (FCInfo), TARGET :: FC
  TYPE (PseudoInfo), TARGET :: PAW

  TYPE (Word_Context) :: wc
  CHARACTER(100)      :: delim
  CHARACTER(100)      :: Returndelim
  CHARACTER           :: comment
  CHARACTER           :: literal
  CHARACTER(100)      :: filename
  CHARACTER(100)     :: token,token2,token3,token4,gridtype,gridid
  INTEGER             :: tlen
  REAL(8)             :: grida, gridd

  EXTERNAL          GetArg
  INTEGER, EXTERNAL :: iArgC

  CALL Init_GlobalConstants

  ! prepare some arrays

  ALLOCATE(PAW%np(maxdim),PAW%l(maxdim),PAW%eig(maxdim),&
       PAW%occ(maxdim),PAW%label(maxdim),STAT=OK)
  IF (OK /= 0) THEN
     WRITE(6,*) 'xmlpaw: error in allocation --', maxdim,OK
     STOP
  ENDIF

  ! prepare to read xml file
  delim = " ="
  returndelim = "</>"
  comment = ACHAR(0)   !** Make the comment a nul character
  Literal = '"'

  CALL GetArg(1,filename)

  WRITE(*,*) 'Processing file : ', TRIM(filename)

  CALL DetWordConstants(20)

  CALL InitWord(wc, 20, filename, delim, returndelim, comment, literal, 5, "No-include")

  PAW%rc=0
  DO WHILE (W_ERROR /= W_EOF)

     CALL GetNExtWord(wc, token, tlen)
     !WRITE(6,*)'output', TRIM(token)

     IF (TRIM(token)=="<") THEN
        CALL GetNextWord(wc,token2,tlen)
        IF (TRIM(token2)=="?xml".OR. &
             TRIM(token2)=="!DOCTYPE".OR. &
             TRIM(token2)=="paw_setup") THEN
           !WRITE(6,*) 'In header', TRIM(token2)
           CALL GetNExtWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           !WRITE(6,*) 'Out header', TRIM(token3)
        ELSEIF (TRIM(token2)=="atom") THEN
           !WRITE(6,*) 'In atom',TRIM(token2)
           DO i=1,4
              CALL GetNextWord(wc,token3,tlen)
              IF (TRIM(token3)=="symbol") CALL Getnextword(wc,AEpot%sym,tlen)
              IF (TRIM(token3)=="Z") CALL Getnumber(wc,AEpot%nz)
              IF (TRIM(token3)=="core") CALL Getnumber(wc,FC%zcore)
              IF (TRIM(token3)=="valence") CALL Getnumber(wc,FC%zvale)
           ENDDO
           WRITE(6,*) 'symbol', AEpot%sym
           WRITE(6,*) 'Z',     AEpot%nz
           WRITE(6,*) 'core',  FC%zcore
           WRITE(6,*) 'valence',  FC%zvale
        ELSEIF (TRIM(token2)=="xc_functional") THEN
           CALL GetNextWord(wc,token3,tlen)
           IF (TRIM(token3)=="type") CALL GetNextWord(wc,exctype,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           CALL initexch
        ELSEIF (TRIM(token2)=="generator") THEN
           !WRITE(6,*) 'In generator', TRIM(token2)
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= "generator")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           CALL GetNextWord(wc,Token3,tlen)
           !WRITE(6,*)'Out generator', TRIM(token2)
        ELSEIF (TRIM(token2)=="ae_energy") THEN
           !WRITE(6,*) 'In ae_energy',TRIM(token2)
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              IF (TRIM(token3)=="kinetic")&
                   CALL GetNumber(wc,AESCF%ekin)
              IF (TRIM(token3)=="xc")&
                   CALL GetNumber(wc,AESCF%eexc)
              IF (TRIM(token3)=="electrostatic")&
                   CALL GetNumber(wc,AESCF%estatic)
              IF (TRIM(token3)=="total")&
                   CALL GetNumber(wc,AESCF%etot)
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           AESCF%ekin=2*AESCF%ekin
           AESCF%eexc=2*AESCF%eexc
           AESCF%estatic=2*AESCF%estatic
           AESCF%etot=2*AESCF%etot
           WRITE(6,*)'Ae kinetic', AESCF%ekin
           WRITE(6,*)'Ae exch', AESCF%eexc
           WRITE(6,*)'Ae electrostatic', AESCF%estatic
           WRITE(6,*)'Ae total', AESCF%etot
        ELSEIF (TRIM(token2)=="core_energy") THEN
           !WRITE(6,*) 'In core_energy',TRIM(token2)
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              IF (TRIM(token3)=="kinetic")&
                   CALL GetNumber(wc,FC%corekin)
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           FC%corekin=2*FC%corekin
           WRITE(6,*)'Core kinetic', FC%corekin
        ELSEIF (TRIM(token2)=="valence_states") THEN
           PAW%nbase=0
           CALL GetNextWord(wc,token3,tlen)
           DO 
              IF (TRIM(token3)=="state") THEN
                 PAW%nbase=PAW%nbase+1
                 PAW%np(PAW%nbase)=999
                 CALL GetNextWord(wc,token4,tlen)
                 DO
                    IF (TRIM(token4)=="n") &
                         CALL GetNumber(wc,PAW%np(PAW%nbase))
                    IF (TRIM(token4)=="l") &
                         CALL GetNumber(wc,PAW%l(PAW%nbase))
                    IF (TRIM(token4)=="f") &
                         CALL GetNumber(wc,PAW%occ(PAW%nbase))
                    IF (TRIM(token4)=="e") &
                         CALL GetNumber(wc,PAW%eig(PAW%nbase))
                    IF (TRIM(token4)=="rc") &
                         CALL GetNumber(wc,rc)
                    PAW%rc=MAX(rc,PAW%rc)
                    IF (TRIM(token4)=="id") &
                         CALL GetNextWord(wc,PAW%label(PAW%nbase),tlen)
                    IF (TRIM(token4)==">") EXIT
                    CALL GetNextWord(wc,token4,tlen)
                 ENDDO
                 CALL GetNextWord(wc,token3,tlen)
              ELSEIF (TRIM(token3)=="valence_states") THEN
                 CALL GetNextWord(wc,token3,tlen)
                 !WRITE(6,*) 'Found basis states:', PAW%nbase
                 DO i=1,PAW%nbase
                    PAW%eig(i)=2*PAW%eig(i)   ! change to Hartree units
                    WRITE(6,&
                         '("np = ",i4," l = ",i4,"  e = ",1pe14.7,"  occ = ",1pe14.7)')&
                         PAW%np(i),PAW%l(i),PAW%eig(i),PAW%occ(i)
                 ENDDO
                 WRITE(6,*) 'rc = ', PAW%rc
                 EXIT
              ELSE
                 CALL GetNextWord(wc,token3,tlen)
              ENDIF
           ENDDO
        ELSEIF (TRIM(token2)=="radial_grid") THEN
           CALL GetNextWord(wc,token3,tlen)
           !WRITE(6,*)'radial grid word',TRIM(token3)
           DO
              IF (TRIM(token3)=="eq") THEN
                 !WRITE(6,*) 'eq'
                 CALL GetNextWord(wc,gridtype,tlen)
              ENDIF
              IF (TRIM(token3)=="d") THEN 
                 !WRITE(6,*) 'd'
                 CALL GetNumber(wc,gridd)
              ENDIF
              IF (TRIM(token3)=="a") THEN
                 !WRITE(6,*) 'a'
                 CALL GetNumber(wc,grida)
              ENDIF
              IF (TRIM(token3)=="i") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 CALL ConvertChar(TRIM(Token4),AEGrid%n)
              ENDIF
              IF (TRIM(token3)=="id") THEN
                 !WRITE(6,*) 'id'
                 CALL GetNextWord(wc,gridid,tlen)
              ENDIF
              IF (TRIM(token3)==">") EXIT
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           WRITE(6,*) 'gridtype',TRIM(gridtype)
           WRITE(6,*) 'gridd', gridd
           WRITE(6,*) 'n', AEGrid%n
           WRITE(6,*)  'gridid', TRIM(gridid)
           IF (TRIM(gridtype)=="r=d*i") THEN
              WRITE(6,*) 'linear grid', TRIM(gridtype)
              AEGrid%h=gridd
              ALLOCATE(AEgrid%r(AEgrid%n),stat=OK)
              IF (OK /= 0) THEN
                 WRITE(6,*) 'xmlpaw: allocation error in AEGrid', AEgrid%n,OK
                 STOP
              ENDIF
              DO i=1,AEGrid%n
                 AEgrid%r(i)=AEGrid%h*i
              ENDDO
              WRITE(6,*) 'Grid contructed with h, n', AEGrid%h,AEGrid%n
              coretailpoints=AEGrid%n
              allocate(dum(AEgrid%n+1),STAT=OK)
                    if (OK /= 0) then
                        write(6,*) 'error in dum allocation', AEGrid%n+1,OK
                        stop
                    endif
           ELSE
              WRITE(6,*) 'Non-linear grid -- more work has to be done'
              WRITE(6,*)  'Program will exit and wait for you to '
              WRITE(6,*)  '     add an interpolation routine'
              STOP
           ENDIF
        ELSEIF (TRIM(token2)=="shape_function") THEN
           CALL GetNextWord(wc,token3,tlen)
           IF (TRIM(token3)=="grid") THEN
              CALL GetNextWord(wc,token4,tlen)
              IF (TRIM(token4)=="lin") THEN
                 CALL GetNextWord(wc,token4,tlen)
              ELSE
                 WRITE(6,*) 'Grid label = ',TRIM(Token4)
                 WRITE(6,*) 'Non-linear grid -- more work has to be done'
                 WRITE(6,*)  'Program will exit and wait for you to '
                 WRITE(6,*)  '     add an interpolation routine'
                 STOP
              ENDIF
           ELSE
              WRITE(6,*) 'shape parameter = ',TRIM(Token3)
              WRITE(6,*) 'Special shape function -- more work has to be done'
              WRITE(6,*)  'Program will exit and wait for you to '
              WRITE(6,*)  '     add appropriate code'
              STOP
           ENDIF
           Call GetNumbers(wc,dum,AEGrid%n+1)
           allocate(PAW%hatshape(AEGrid%n),STAT=OK)
                    if (OK /= 0) then
                        write(6,*) 'error in hatshape allocation', AEGrid%n+1,OK
                        stop
                    endif
           PAW%hatshape(1:AEgrid%n)=dum(2:AEgrid%n+1)
           !do i=1,AEgrid%n
           !    write(100,*) AEgrid%r(i),PAW%hatshape(i)
           !enddo
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
        ELSEIF (TRIM(token2)=="ae_core_density") THEN
           CALL GetNextWord(wc,token3,tlen)
           IF (TRIM(token3)=="grid") THEN
              CALL GetNextWord(wc,token4,tlen)
              IF (TRIM(token4)=="lin") THEN
                 CALL GetNextWord(wc,token4,tlen)
              ELSE
                 WRITE(6,*) 'Grid label = ',TRIM(Token4)
                 WRITE(6,*) 'Non-linear grid -- more work has to be done'
                 WRITE(6,*)  'Program will exit and wait for you to '
                 WRITE(6,*)  '     add an interpolation routine'
                 STOP
              ENDIF
           ELSE
              WRITE(6,*) 'bad parameter = ',TRIM(Token3)
              WRITE(6,*) 'Program exiting due to typo'
              STOP
           ENDIF
           Call GetNumbers(wc,dum,AEGrid%n+1)
           allocate(FC%coreden(AEGrid%n),STAT=OK)
                    if (OK /= 0) then
                        write(6,*) 'error in coreden allocation', AEGrid%n+1,OK
                        stop
                    endif
           FC%coreden(1:AEgrid%n)=dum(2:AEgrid%n+1)*AEGrid%r(1:AEgrid%n)
           !do i=1,AEgrid%n
           !    write(102,*) AEgrid%r(i),FC%coreden(i)
           !enddo
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
        ELSEIF (TRIM(token2)=="pseudo_core_density") THEN
           CALL GetNextWord(wc,token3,tlen)
           IF (TRIM(token3)=="grid") THEN
              CALL GetNextWord(wc,token4,tlen)
              IF (TRIM(token4)=="lin") THEN
                 CALL GetNextWord(wc,token4,tlen)
              ELSE
                 WRITE(6,*) 'Grid label = ',TRIM(Token4)
                 WRITE(6,*) 'Non-linear grid -- more work has to be done'
                 WRITE(6,*)  'Program will exit and wait for you to '
                 WRITE(6,*)  '     add an interpolation routine'
                 STOP
              ENDIF
           ELSE
              WRITE(6,*) 'bad parameter = ',TRIM(Token3)
              WRITE(6,*) 'Program exiting due to typo'
              STOP
           ENDIF
           Call GetNumbers(wc,dum,AEGrid%n+1)
           allocate(PAW%tcore(AEGrid%n),STAT=OK)
                    if (OK /= 0) then
                        write(6,*) 'error in tcore allocation', AEGrid%n+1,OK
                        stop
                    endif
           PAW%tcore(1:AEgrid%n)=dum(2:AEgrid%n+1)*AEGrid%r(1:AEgrid%n)
           !do i=1,AEgrid%n
           !    write(103,*) AEgrid%r(i),PAW%tcore(i)
           !enddo
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
        ELSEIF (TRIM(token2)=="localized_potential") THEN
           CALL GetNextWord(wc,token3,tlen)
           IF (TRIM(token3)=="grid") THEN
              CALL GetNextWord(wc,token4,tlen)
              IF (TRIM(token4)=="lin") THEN
                 CALL GetNextWord(wc,token4,tlen)
              ELSE
                 WRITE(6,*) 'Grid label = ',TRIM(Token4)
                 WRITE(6,*) 'Non-linear grid -- more work has to be done'
                 WRITE(6,*)  'Program will exit and wait for you to '
                 WRITE(6,*)  '     add an interpolation routine'
                 STOP
              ENDIF
           ELSE
              WRITE(6,*) 'bad parameter = ',TRIM(Token3)
              WRITE(6,*) 'Program exiting due to typo'
              STOP
           ENDIF
           Call GetNumbers(wc,dum,AEGrid%n+1)
           allocate(PAW%vloc(AEGrid%n),STAT=OK)
                    if (OK /= 0) then
                        write(6,*) 'error in vloc allocation', AEGrid%n+1,OK
                        stop
                    endif
           PAW%vloc(1:AEgrid%n)=2*dum(2:AEgrid%n+1)*AEGrid%r(1:AEgrid%n)
           !do i=1,AEgrid%n
           !    write(104,*) AEgrid%r(i),PAW%vloc(i)
           !enddo
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
        ELSEIF (TRIM(token2)=="ae_partial_wave") THEN
           ! check if arrays have been allocated
           IF (.NOT.Associated(PAW%ophi)) THEN
              ALLOCATE(PAW%ophi(AEgrid%n,PAW%nbase),STAT=OK)
              IF (OK /= 0) THEN
                 WRITE(6,*) 'Allocation error ophi', AEgrid%n,PAW%nbase
                 STOP
              ENDIF
              PAW%ophi=0
           ENDIF
           CALL GetNextWord(wc,token3,tlen)
           DO
           !  write(6,*) 'In ae_partial_wave',trim(token3)
              IF (TRIM(token3)=="grid") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 IF (TRIM(token4)=="lin") THEN
                    CALL GetNextWord(wc,token3,tlen)
                 ELSE
                    WRITE(6,*) 'Grid label = ',TRIM(Token4)
                    WRITE(6,*) 'Non-linear grid -- more work has to be done'
                    WRITE(6,*)  'Program will exit and wait for you to '
                    WRITE(6,*)  '     add an interpolation routine'
                    STOP
                 ENDIF
              ELSEIF (TRIM(token3)=="state") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 j=0
                 DO i=1,PAW%nbase
           !         write(6,*) 'label',TRIM(token4),TRIM(PAW%label(i))
                    IF (TRIM(token4)==TRIM(PAW%label(i))) j=i
                 ENDDO
           !      write(6,*)'In ae_partial_wave ',j
                 IF (j<=0) THEN
                    WRITE(6,*) 'Error in state label' , TRIM(token4)
                    stop
                 ENDIF
                 CALL GetNextWord(wc,token3,tlen)
              ELSEIF (TRIM(token3)==">") then
                 exit
              ELSE
                 CALL GetNextWord(wc,token3,tlen)
              ENDIF
           ENDDO
              CALL GetNumbers(wc,dum,AEgrid%n+1)
              PAW%ophi(1:AEgrid%n,j)=dum(2:AEgrid%n+1)*AEgrid%r(1:AEgrid%n)
              !do i=1,AEgrid%n
              !   write(200+j,*) AEgrid%r(i),PAW%ophi(i,j)
              !enddo
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              !write(6,*) 'after ae_partial',trim(token3)
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
           !write(6,*)'after ae_partial',trim(token2),trim(token3)
        ELSEIF (TRIM(token2)=="pseudo_partial_wave") THEN
           !write(6,*) 'In pseudo_partial_wave'
           ! check if arrays have been allocated
           IF (.NOT.Associated(PAW%otphi)) THEN
              ALLOCATE(PAW%otphi(AEgrid%n,PAW%nbase),STAT=OK)
              IF (OK /= 0) THEN
                 WRITE(6,*) 'Allocation error ophi', AEgrid%n,PAW%nbase
                 STOP
              ENDIF
              PAW%otphi=0
           ENDIF
           CALL GetNextWord(wc,token3,tlen)
           DO
              IF (TRIM(token3)=="grid") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 IF (TRIM(token4)=="lin") THEN
                    CALL GetNextWord(wc,token3,tlen)
                 ELSE
                    WRITE(6,*) 'Grid label = ',TRIM(Token4)
                    WRITE(6,*) 'Non-linear grid -- more work has to be done'
                    WRITE(6,*)  'Program will exit and wait for you to '
                    WRITE(6,*)  '     add an interpolation routine'
                    STOP
                 ENDIF
              ELSEIF (TRIM(token3)=="state") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 j=0
                 DO i=1,PAW%nbase
                   ! write(6,*) 'label',TRIM(token4),TRIM(PAW%label(i))
                    IF (TRIM(token4)==TRIM(PAW%label(i))) j=i
                 ENDDO
                 IF (j<=0) THEN
                    WRITE(6,*) 'Error in state label' , TRIM(token4)
                    stop
                 ENDIF
                 CALL GetNextWord(wc,token3,tlen)
              ELSEIF (TRIM(token3)==">") Then
                 Exit
              ELSE
                 CALL GetNextWord(wc,token3,tlen)
              ENDIF
           ENDDO
           !write(6,*)'In pseudo_partial_wave ',j
           CALL GetNumbers(wc,dum,AEgrid%n+1)
           PAW%otphi(1:AEgrid%n,j)=dum(2:AEgrid%n+1)*AEgrid%r(1:AEgrid%n)
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
        ELSEIF (TRIM(token2)=="projector_function") THEN
           ! check if arrays have been allocated
           IF (.NOT.Associated(PAW%otp)) THEN
              ALLOCATE(PAW%otp(AEgrid%n,PAW%nbase),STAT=OK)
              IF (OK /= 0) THEN
                 WRITE(6,*) 'Allocation error ophi', AEgrid%n,PAW%nbase
                 STOP
              ENDIF
              PAW%otp=0
           ENDIF
           CALL GetNextWord(wc,token3,tlen)
           DO
              IF (TRIM(token3)=="grid") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 IF (TRIM(token4)=="lin") THEN
                    CALL GetNextWord(wc,token3,tlen)
                 ELSE
                    WRITE(6,*) 'Grid label = ',TRIM(Token4)
                    WRITE(6,*) 'Non-linear grid -- more work has to be done'
                    WRITE(6,*)  'Program will exit and wait for you to '
                    WRITE(6,*)  '     add an interpolation routine'
                    STOP
                 ENDIF
              ELSEIF (TRIM(token3)=="state") THEN
                 CALL GetNextWord(wc,token4,tlen)
                 j=0
                 DO i=1,PAW%nbase
                  !  write(6,*) 'label',TRIM(token4),TRIM(PAW%label(i))
                    IF (TRIM(token4)==TRIM(PAW%label(i))) j=i
                 ENDDO
                 IF (j<=0) THEN
                    WRITE(6,*) 'Error in state label' , TRIM(token4)
                    stop
                 ENDIF
                 CALL GetNextWord(wc,token3,tlen)
              ELSEIF (TRIM(token3)==">") THEN
                 exit
              ELSE
                 CALL GetNextWord(wc,token3,tlen)
              ENDIF
           ENDDO
           !write(6,*)'In projector_function ',j
           CALL GetNumbers(wc,dum,AEgrid%n+1)
           PAW%otp(1:AEgrid%n,j)=dum(2:AEgrid%n+1)*AEgrid%r(1:AEgrid%n)
           CALL GetNextWord(wc,token3,tlen)
           DO WHILE (TRIM(token3) /= ">")
              CALL GetNextWord(wc,token3,tlen)
           ENDDO
        ELSEIF (TRIM(token2)=="kinetic_energy_differences") THEN
           ALLOCATE(PAW%kin(PAW%nbase,PAW%nbase),STAT=OK)
              IF (OK /= 0) THEN
                 WRITE(6,*) 'Allocation error kin', PAW%nbase, OK
                 STOP
              ENDIF
           PAW%kin=0
           Do i = 1, PAW%nbase
              CALL GetNumbers(wc,PAW%kin(i,:),PAW%nbase)
           enddo
           CALL GetNextWord(wc,token3,tlen)
           Do 
             If (TRIM(token3)==">") EXIT
             CALL GetNextWord(wc,token3,tlen)
           enddo


        ENDIF
     ENDIF
     WRITE(*,*) TRIM(token)
  END DO

  CALL CloseWord(wc)

  write(6,*) 'Completed reading in xml file'

  OPEN(ifen,file=TRIM(AEPot%sym)//'.fromxml',form='formatted')

  WRITE(ifen,'("Atom = ",a2,"  Z = ",i4)')  AEPot%sym, AEPot%nz
  SELECT CASE(TRIM(exctype))
  CASE default
     WRITE(6,*) 'Perdew-Wang correlation'
     WRITE(ifen,*) 'Perdew-Wang correlation'
  CASE('LDA-PW')
     WRITE(6,*) 'Perdew-Wang correlation'
     WRITE(ifen,*) 'Perdew-Wang correlation'
  CASE('GGA-PBE')
     WRITE(6,*) 'Perdew - Burke - Ernzerhof GGA'
     WRITE(ifen,*) 'Perdew - Burke - Ernzerhof GGA'
  END SELECT

  n=>AEGrid%n
  h=>AEGrid%h
  r=>AEGrid%r
  rv=>AEPot%rv
  WRITE(ifen,'("Integration params -- n,rmax = ",i5,1p2e15.7)') &
       AEGrid%n,AEGrid%r(n)
  WRITE(ifen,*) ' core charge (zcore) = ',FC%zcore
  WRITE(ifen,*) ' valence charge (zvale) = ',FC%zvale
  ! find irc
  irc=PAW%rc/h+0.5 ; PAW%irc=irc
  rc=PAW%rc
  Write(6,*) 'rc-irc*h', PAW%rc-h*irc
  WRITE(6,*) ' adjusted rc ',rc, r(irc)
  WRITE(6,*) 'irc,rc = ',irc,rc
  WRITE(ifen,*) ' paw parameters: '
  WRITE(ifen,*) '        rc = ',rc
  WRITE(ifen,*) '       irc = ',irc
  !
  OPEN(ifout,file='coredensity.fromxml', form='formatted')
  DO i=1,n
     IF (FC%coreden(i)<machine_zero) FC%coreden(i)=0
     IF (PAW%tcore(i)<machine_zero) PAW%tcore(i)=0
     WRITE(ifout,'(1p1e15.7,1p4e25.17)') r(i),FC%coreden(i),&
          PAW%tcore(i)
  ENDDO
  CLOSE(ifout)

  OPEN(ifout,file='vloc.fromxml', form='formatted')
  DO i=1,irc
     WRITE(ifout,'(1p1e15.7,1p3e25.17)') r(i),PAW%vloc(i)
  ENDDO
  CLOSE(ifout)

  nbase=PAW%nbase
  WRITE(ifen,'(/"Number of basis functions ",i5)') nbase
  WRITE(ifen,*)'No.   n    l      Energy         Occ'

  DO io=1,nbase
     WRITE(ifen,'(3i5,1p3e15.7)') io,PAW%np(io),PAW%l(io),PAW%eig(io),&
          PAW%occ(io)
     CALL mkname(io,flnm)
     OPEN(ifout,file='wfn'//TRIM(flnm)//'.fromxml',form='formatted')
     WRITE(ifout,*) 'l=',PAW%l(io),'basis function with energy  ',PAW%eig(io)
       DO i=1,irc
          WRITE(ifout,'(1p5e12.4)') r(i),PAW%ophi(i,io),PAW%otphi(i,io),PAW%otp(i,io)
       ENDDO
       CLOSE(ifout)
  ENDDO

    !Begin  constructing output  for pwpaw pgm

    qcore=FC%zcore
    OPEN(ifatompaw,file=TRIM(AEpot%sym)//'.atomicdata.fromxml',form='formatted')
    WRITE(ifatompaw,'("  ATOMTYPE     ",a2)') AEpot%sym
    WRITE(ifatompaw,'("  ATOMXCTYPE     ",a10)') exctype
    WRITE(ifatompaw,'("  ATOMIC_CHARGE    ",i5)') AEpot%nz
    WRITE(ifatompaw,'("  CORE_CHARGE    ",1pe20.13)') qcore
    WRITE(ifatompaw,'("  RC         ",1pe20.13)') PAW%rc
    WRITE(ifatompaw,'("  SHAPE_TYPE  ",a20)') 'sinc2'

    WRITE(ifatompaw,'("  BASIS_SIZE    ",i5)') PAW%nbase
    WRITE(ifatompaw,'("  ORBITALS      ")')
    WRITE(ifatompaw,'(20i4)') (PAW%l(ib),ib=1,nbase)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("  MESH_SIZE    ",i10)') PAW%irc+1
    WRITE(ifatompaw,'("  MESH_STEP    ",1pe20.13)') AEGrid%h

    WRITE(ifatompaw,'("  CORETAIL_POINTS   ",i10)') coretailpoints+1


    !tphirange=10.d0       ! tphirange=8.d0
    !lcao_points=800       ! lcao_points=600
    !lcao_i=tphirange/AEGrid%h
    !iskip=MAX(1,lcao_i/lcao_points)
    !lcao_i=iskip*(lcao_i/iskip)+iskip
    !hlcao=AEGrid%h*iskip
    !lcao_points=lcao_i/iskip + 1
    lcao_points=coretailpoints+1
    hlcao=AEGrid%h
    WRITE(ifatompaw,'("  LCAO_SIZE  ",i10)') lcao_points
    WRITE(ifatompaw,'("  LCAO_STEP   ",1pe20.13)') hlcao

    WRITE(ifatompaw,'("   CORE_DENSITY   ")')
    WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(FC%coreden(i),i=1,PAW%irc)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("   CORETAIL_DENSITY   ")')
    WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%tcore(i),i=1,coretailpoints)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("   SHAPE_FUNC   ")')
    WRITE(ifatompaw,'(1p3e25.17)') 1.d0,(PAW%hatshape(i),i=1,PAW%irc)
    WRITE(ifatompaw,'("  END     ")')

    WRITE(ifatompaw,'("   VLOCFUN      ")')
    x=PAW%vloc(3)+3*(PAW%vloc(1)-PAW%vloc(2))
    WRITE(ifatompaw,'(1p3e25.17)') x,(PAW%vloc(i),i=1,PAW%irc)
    WRITE(ifatompaw,'("  END     ")')

    DO ib=1,nbase
       WRITE(ifatompaw,'("   TPROJECTOR",i4," #p(r), for p(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%otp(i,ib),i=1,PAW%irc)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO

    DO ib=1,nbase
       WRITE(ifatompaw,'("   PHI",i5," #phi(r), for phi(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%ophi(i,ib),i=1,PAW%irc)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO

    DO ib=1,nbase
       WRITE(ifatompaw,'("   TPHI",i5," #tphi(r), for tphi(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(PAW%otphi(i,ib),i=1,PAW%irc)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO

! add quadratic tail for lcao function

    ALLOCATE(ttphi(n),PAW%hatden(n),PAW%hatpot(n),PAW%den(n),&
          PAW%tden(n),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating ttphi',n,ok
       STOP
    ENDIF

    DO ib=1,nbase
       ttphi=PAW%otphi(:,ib)
       x=ttphi(irc)/(rc-r(coretailpoints))**2
       write(6,*) 'For ib', ib ,'  quadratic fac = ', x
       do i=irc+1,coretailpoints
          ttphi(i)=x*(r(i)-r(coretailpoints))**2
       enddo
       WRITE(ifatompaw,'("   TPHI_LCAO",i4," #tphi0(r) for tphi0(r)/r*Ylm)")') ib
       WRITE(ifatompaw,'(1p3e25.17)') 0.d0,(ttphi(j),j=1,coretailpoints)
       WRITE(ifatompaw,'("  END     ")')
    ENDDO
  
    !set hatden
      PAW%hatden=0
      PAW%hatden(1:irc)=PAW%hatshape(1:irc)*(r(1:irc)**2)
                                                                                
      !  normalize
      con=overint(n,h,PAW%hatden)
      WRITE(6,*) ' check hatden normalization', con
      PAW%hatden=PAW%hatden/con


    ! spherical matrix elements
    icount=(nbase*(nbase+1))/2
    ALLOCATE(soij(icount),stij(icount),svij(icount),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating dia matrix elements',icount,ok
       STOP
    ENDIF

    ! allocate PAW data set storage for spherical matrix elements
    ALLOCATE(PAW%tvij(nbase,nbase),PAW%vhatij(nbase,nbase),PAW%v0ij(nbase,nbase),&
    !     PAW%kin(nbase,nbase),PAW%vhijkl(nbase,nbase,nbase,nbase),stat=ok)
         PAW%vhijkl(nbase,nbase,nbase,nbase),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating storage for matrix elements',nbase,ok
       STOP
    ENDIF
    PAW%tvij=0; PAW%vhatij=0; PAW%v0ij=0; PAW%vhijkl=0

    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          IF (PAW%l(ib)==PAW%l(ic)) THEN
             l=PAW%l(ib)
             icount=icount+1
             CALL dqij(AEGrid,PAW,ib,ic,soij(icount))
             CALL dtij(AEGrid,PAW,ib,ic,stij(icount))
             CALL dvij(AEGrid,PAW,FC,AEPot%nz,ib,ic,svij(icount))
             PAW%tvij(ib,ic)=stij(icount)+svij(icount)
             PAW%tvij(ic,ib)=stij(icount)+svij(icount)
          ENDIF
       ENDDO
    ENDDO

    WRITE(ifatompaw,'("   OVERLAP_SIZE    ",i10)') icount
    WRITE(ifatompaw,'("   OVERLAP_MATRIX  ")')
    WRITE(ifatompaw,'(1p3e25.17)') (soij(ic),ic=1,icount)
    WRITE(ifatompaw,'("  END     ")')
    WRITE(ifatompaw,'("   KINETIC_ENERGY_MATRIX  ")')
    WRITE(ifatompaw,'(1p3e25.17)') (stij(ic),ic=1,icount)
    WRITE(ifatompaw,'("  END     ")')
    WRITE(ifatompaw,'("   V_ION_MATRIX  ")')
    WRITE(ifatompaw,'(1p3e25.17)') (svij(ic),ic=1,icount)
    WRITE(ifatompaw,'("  END     ")')

    !
    !  angularly dependent matrix elements
    !
    icount=(nbase*(nbase+1))*(PAW%lmax+1)
    ALLOCATE(shij(icount),snij(icount),stat=ok)
    IF (ok /= 0) THEN
       WRITE(6,*) 'Error in allocating L matrix elements',icount,ok
       STOP
    ENDIF

    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             icount=icount+1
             DO i=1,irc
                rr=h*i
                dum(i)=(rr**ll)*(PAW%ophi(i,ib)*PAW%ophi(i,ic) &
                     -PAW%otphi(i,ib)*PAW%otphi(i,ic))
             ENDDO
             snij(icount)=overint(irc,h,dum)
             IF (ll==0)  THEN
                PAW%v0ij(ib,ic)=snij(icount)
                PAW%v0ij(ic,ib)=snij(icount)
             ENDIF
             CALL hatpotL(AEGrid,PAW,ll,dum)
             DO i=1,irc
                dum(i)=dum(i)*PAW%otphi(i,ib)*PAW%otphi(i,ic)
             ENDDO
             shij(icount)=overint(irc,h,dum)
             IF (ll==0)  THEN
                PAW%vhatij(ib,ic)=shij(icount)
                PAW%vhatij(ic,ib)=shij(icount)
             ENDIF
          ENDDO
       ENDDO
    ENDDO

    WRITE(ifatompaw,'("   DENVHAT_SIZE   ",i10)') icount
    WRITE(ifatompaw,'("   DENSITY   ")')
    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             icount=icount+1
             WRITE(ifatompaw,'(3i10,1pe25.17)') ib,ic,ll,snij(icount)
          ENDDO
       ENDDO
    ENDDO
    WRITE(ifatompaw,'("   END   ")')
    WRITE(ifatompaw,'("   V_HAT   ")')
    icount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             icount=icount+1
             WRITE(ifatompaw,'(3i10,1pe25.17)') ib,ic,ll,shij(icount)
          ENDDO
       ENDDO
    ENDDO
    WRITE(ifatompaw,'("   END   ")')
    !
    icount=(nbase*(nbase+1))/2
    lcount=(icount**2)*((PAW%lmax+1)*2)
    ALLOCATE(shartree(lcount),sshartree(icount,icount,2*PAW%lmax+2),&
         wf(n),twf(n),rh(n),rth(n),stat=ok)
    IF (ok/=0) THEN
       WRITE(6,*) 'Error in hartree allocation', icount,lcount,irc,ok
       STOP
    ENDIF
    !
    !  Hartree matrix elements
    !
    !! Due to precision error in apoisson solver, some equivalent Hartree
    !!  matrix elements are not equal with an error of e-5; take average
    !!  these elements in order to remove assymmetry errors
    !!  Thanks to Francois Jollet for pointing out this problem
    sshartree=0
    shartree=0
    icount=0
    lcount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          icount=icount+1
          DO i=1,irc
             wf(i)=PAW%ophi(i,ib)*PAW%ophi(i,ic)
             twf(i)=PAW%otphi(i,ib)*PAW%otphi(i,ic)
          ENDDO
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             CALL apoisson(ll,irc,h,wf,rh)
             CALL apoisson(ll,irc,h,twf,rth)
             jcount=0
             DO id=1,nbase
                DO ie=id,nbase
                   jcount=jcount+1
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      !WRITE(6,*) 'ib,ic,ll',ib,ic,id,ie,ll,lcount,icount,jcount
                      DO i=1,irc
                         rr=h*i
                         dum(i)=PAW%ophi(i,id)*PAW%ophi(i,ie)*rh(i)/rr            &
                              -PAW%otphi(i,id)*PAW%otphi(i,ie)*rth(i)/rr
                      ENDDO
                      shartree(lcount)=overint(irc,h,dum(1:irc))
                      sshartree(icount,jcount,ll+1)=shartree(lcount)
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO

    icount=0
    lcount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          icount=icount+1
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             jcount=0
             DO id=1,nbase
                DO ie=id,nbase
                   jcount=jcount+1
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      shartree(lcount)=                      &
                           0.5d0*(sshartree(icount,jcount,ll+1) + &
                           sshartree(jcount,icount,ll+1))
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO

    WRITE(6,*) lcount,'   ; # Hartree matrix elements'
    WRITE(ifatompaw,'("   HARTREE_SIZE   ",i10)') lcount
    WRITE(ifatompaw,'("   V_HARTREE   ")')
    lcount=0
    DO ib=1,nbase
       DO ic=ib,nbase
          llmin=ABS(PAW%l(ib)-PAW%l(ic))
          llmax=PAW%l(ib)+PAW%l(ic)
          DO ll=llmin,llmax,2
             DO id=1,nbase
                DO ie=id,nbase
                   IF (ll.GE.ABS(PAW%l(id)-PAW%l(ie)).AND.           &
                        ll.LE.PAW%l(id)+PAW%l(ie)) THEN
                      lcount=lcount+1
                      WRITE(ifatompaw,'(5i5,1pe25.17)')ib,ic,id,ie,ll,shartree(lcount)
                      IF (ll==0) THEN
                         PAW%vhijkl(ib,ic,id,ie)=shartree(lcount)
                         PAW%vhijkl(ic,ib,id,ie)=shartree(lcount)
                         PAW%vhijkl(ib,ic,ie,id)=shartree(lcount)
                         PAW%vhijkl(ic,ib,ie,id)=shartree(lcount)
                         PAW%vhijkl(id,ie,ib,ic)=shartree(lcount)
                         PAW%vhijkl(id,ie,ic,ib)=shartree(lcount)
                         PAW%vhijkl(ie,id,ib,ic)=shartree(lcount)
                         PAW%vhijkl(ie,id,ic,ib)=shartree(lcount)
                      ENDIF
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    WRITE(ifatompaw,'("   END         ")')
    
    PAW%lmax=0
    Do ib=1,nbase
       if (PAW%l(ib)>PAW%lmax) PAW%lmax=PAW%l(ib)
    enddo

    WRITE(ifatompaw,'("   HAT_SELF-ENERGY    ",i10)') 2*PAW%lmax
    DO ll=0,2*PAW%lmax
       CALL selfhatpot(AEGrid,PAW,ll,eself)
       WRITE(ifatompaw,'(i10,1pe25.17)') ll,eself
       IF (ll==0) storeself=eself
    ENDDO
    WRITE(ifatompaw,'("   END   ")')

    lcount=(nbase*(nbase+1))/2
    ALLOCATE(bm(nbase),cm(nbase),dm(lcount,lcount),stat=ok)
    IF (ok/=0) THEN
       WRITE(6,*) 'Error in allocation before evale', nbase,ok
       STOP
    ENDIF

    CALL coretailselfenergy(AEGrid,PAW,ctctse,cthatse)
    WRITE(ifatompaw,'("   CORETAILSELFENERGY   ",1pe25.17,"  END")') ctctse
    WRITE(ifatompaw,'("   CORETAILHATENERGY   ",1pe25.17,"  END")') cthatse

    !
    !  find evale from ae_energies and core electron density
    !     evale=ae_energy_total- core_energy_kinetic - 
    !          core-nuclear - core-core
    !

    dum=-2*AEPot%nz*FC%coreden/AEgrid%r
    x=overint(n,h,dum)
    q=FC%zcore
    CALL poisson(n,h,q,FC%coreden,dum,ecore)
    evale=AESCF%etot-FC%corekin-x-ecore
    WRITE(ifen,'("evale from xml info ",1pe25.17)') evale
    WRITE(6,'("evale form xml info",1pe25.17)') evale

    WRITE(ifatompaw,'("   ENERGY   ",1pe25.17,"  END")') evale

    !CALL ftprod(AEGrid,PAW)
    !CALL logderiv(AEGrid,AEPot,PAW)
    !CALL fthatpot(AEGrid,PAW)
    !CALL ftkin(AEGrid,PAW)
    !CALL ftvloc(AEGrid,PAW)
    !
  END PROGRAM xmlpaw
